# ---------------------
#  Model Description
# ---------------------

# Simplified model of an expansion problem.
# The extensions are loops of continuous length
# and the diameters to be looped are the original diameters of the pipes.
# Note, that here we only account for single loops
# and that no extreme points of the convex envelope
# of the cost factors over 1/(equivalent diam)**5 are
# considered.



# -------------
#   Read Data
# -------------

param vertices := "belgium.vertices.csv";
param arcs     := "belgium.arcs.csv";

# ----------
#   Sets
# ----------

set V := { read vertices as "<1s>" comment "#" };
set A := { read arcs as "<1n>" comment "#" };

param numLoops := 1; # number of loops
set LoopIdx := { 1 to numLoops };

# all pipes are allowed to be looped
set Aloops := A*LoopIdx;

param scaling  := 2; # scaling parameter for nomination

# ------------------
#    Functions
# ------------------

# Calculation of an equivalent diameter for k loops with original diameter
defnumb eqdiam(dorig, k) := dorig * exp( 2/5 * ln(k+1) );


# ---------------
#    Parameter
# ---------------

param demand[V] := read vertices as "<1s> 2n" comment "#";
param pMin[V] := read vertices as "<1s> 3n" comment "#"; # bar
param pMax[V] := read vertices as "<1s> 4n" comment "#"; # bar

param lenOrig[A] := read arcs as "<1n> 4n" comment "#"; # km
param diamOrig[A] := read arcs as "<1n> 5n" comment "#"; # m

param tail[A] := read arcs as "<1n> 2s" comment "#";
param head[A] := read arcs as "<1n> 3s" comment "#";

param T      := 281.15;   # gas temperature (K)
param r      :=   0.05;   # absolute rugosity (mm)
param den    :=   0.616;  # density of gas relative to air (-)
param comp   :=   0.8;    # compressibility factor (-)

param lam[<a> in A] := 1/(2 * log(3.71e3 * diamOrig[a] / r) / log(10)) ** 2;
param alpha[<a> in A] :=  (lam[a] * comp * T * lenOrig[a] * den) / ( 96.074830 * (diamOrig[a] ** 5));
param commonfactor[<a> in A] :=  (lam[a] * comp * T * lenOrig[a] * den) / 96.074830;  # corresponds to L*C


# get pressure bounds
param piup[<v> in V] := pMax[v] ** 2;
param pilo[<v> in V] := pMin[v] ** 2;

# get flow bounds
param qlo[<a> in A] := if (piup[head[a]] > pilo[tail[a]])
		       then -sqrt(abs( (numLoops+1) / alpha[a] * (piup[head[a]]  - pilo[tail[a]])))
		       else  sqrt(abs( (numLoops+1) / alpha[a] * (piup[head[a]]  - pilo[tail[a]])))
		       end;
param qup[<a> in A] := if (piup[tail[a]] > pilo[head[a]])
		       then  sqrt(abs((numLoops+1) / alpha[a] * (piup[tail[a]] - pilo[head[a]])))
		       else -sqrt(abs((numLoops+1) / alpha[a] * (piup[tail[a]] - pilo[head[a]])))
		       end;

# get y bounds
param ylo[<a> in A] := commonfactor[a] / eqdiam(diamOrig[a], numLoops)**5;
param yup[<a> in A] := commonfactor[a] / diamOrig[a]**5;

# ---------------
#    Variables
# ---------------

# length variables for each pipe
var lenUnlooped[<a> in A] real  >= 0 <= 1;
var len[<a,i> in Aloops] real >= 0 <= 1;

# weymouth variable
var y[<a> in A] real >= ylo[a] <= yup[a];

# squared pressure vars
var pi[<v> in V] real >= pMin[v]^2 <= pMax[v]^2;

# flow vars
var q[<a> in A] real >= qlo[a] <= qup[a];

# auxiliary vars
var z[<a> in A] real >= -(qlo[a] ** 2) <= (qup[a] ** 2);


# ------------------
#    Constraints
# ------------------

# Original pipe length equals unlooped + looped part
subto coverpipelength: forall <a>  in A do
  lenUnlooped[a] + sum <i> in LoopIdx: len[a,i] == 1;

# Compute weymouth variable
subto weymouth: forall <a>  in A:
  y[a] == commonfactor[a] * ( lenUnlooped[a]/ diamOrig[a]** 5 + sum <i> in LoopIdx: len[a,i] / eqdiam(diamOrig[a], i)**5 );

# Flow conservation
subto flowconservation:  forall <v> in V:
    sum <a> in A with head[a] == v: q[a]
    - sum <a> in A with tail[a] == v: q[a] == scaling * demand[v];

# Auxiliary constraint
subto auxconstr: forall <a> in A:
  z[a] == sgnpow(q[a], 2);

# Pressure Loss constraint
subto pressure_loss: forall <a> in A do
  pi[tail[a]] - pi[head[a]] == y[a] * z[a];



# ------------------------
#    Objective Function
# ------------------------

minimize cost:
sum <a,i> in Aloops:
    len[a,i] * lenOrig[a] * 100 * i;
