
# Simplified model of an expansion planning problem.
# The extensions are loops of discrete length
# and the number of loops is chosen by the model.
# Note, that the number of loops for a single pipe can
# be specified by the following parameter: numLoops.



# --------------
#    Instance
# --------------

set V := { read "belgium.vertices.csv" as "<1s>" comment "#" };
set A := { read "belgium.arcs.csv" as "<1n>" comment "#" };

# ----------------
#    Parameters
# ----------------

param numLoops := 1; # number of loops
param scaling  := 2; # scaling parameter for nomination

# ----------
#    Sets
# ----------

set LoopIdx := { 0 to numLoops };

# ----------------
#    Parameters
# ----------------

# from instance
param demand[V]:= read "belgium.vertices.csv" as "<1s> 2n" comment "#";
param plo[V]   := read "belgium.vertices.csv" as "<1s> 3n" comment "#";
param pup[V]   := read "belgium.vertices.csv" as "<1s> 4n" comment "#";

param tail[A]  := read "belgium.arcs.csv" as "<1n> 2s" comment "#";
param head[A]   := read "belgium.arcs.csv" as "<1n> 3s" comment "#";
param L[A]     := read "belgium.arcs.csv" as "<1n> 4n" comment "#";
param D[A]     := read "belgium.arcs.csv" as "<1n> 5n" comment "#";

# constants from [De Wolf, 2000]
param T      := 281.15;   # gas temperature (K)
param r      :=   0.05;   # absolute rugosity (mm)
param den    :=   0.616;  # density of gas relative to air (-)
param comp   :=   0.8;    # compressibility factor (-)

# computed coefficients for pressure loss equation
param lam[<a> in A] := 1/(2 * log(3.71e3 * D[a] / r) / log(10)) ** 2;
param alpha[<a> in A] := (lam[a] * comp * T * L[a] * den) / (96.074830 * (D[a] ** 5));

# implied bounds for pressure squares pi
param piup[<i> in V] := pup[i] ** 2;
param pilo[<i> in V] := plo[i] ** 2;

# implied bounds for arc flow q
param qlo[<a> in A] := if (piup[head[a]] > pilo[tail[a]])
               then -sqrt(abs(numLoops / alpha[a] * (piup[head[a]]  - pilo[tail[a]])))
               else  sqrt(abs(numLoops / alpha[a] * (piup[head[a]]  - pilo[tail[a]])))
		       end; 
param qup[<a> in A] := if (piup[tail[a]] > pilo[head[a]])
               then  sqrt(abs(numLoops / alpha[a] * (piup[tail[a]] - pilo[head[a]])))
               else -sqrt(abs(numLoops / alpha[a] * (piup[tail[a]] - pilo[head[a]])))
		       end;


# ---------------
#    Variables
# ---------------

var          q[<a> in A]              real >= qlo[a]    <= qup[a];                     # arc flow
var          z[<a> in A]              real >= -infinity <= infinity;                   # squared arc flow
var      delta[<a,i> in A*LoopIdx]    real >= -infinity <= infinity;                   # pressure difference
var         pi[<i> in V]              real >= pilo[i]   <= piup[i];                    # squared pressure
var          x[<a,i> in A*LoopIdx]    binary;                                          # build new loop?



# ------------------------
#    Objective Function
# ------------------------

minimize cost: 
  sum <a,i> in A*LoopIdx do i * 100 * L[a] * x[a,i];

# ------------------
#    Constraints
# ------------------

subto flowconservation: forall <i> in V do
   sum <a> in A with tail[a] == i do q[a] == sum <a> in A with head[a] == i do q[a] - scaling * demand[i];

subto weymouth: forall <a> in A do
   z[a] == sgnpow(q[a], 2)
   and
   z[a] == sum <i> in LoopIdx do (i+1) * (i+1) * delta[a, i]
   and
   sum <i> in LoopIdx do alpha[a] * delta[a, i] ==  pi[tail[a]] - pi[head[a]];
   
subto newloopsum: forall <a> in A do
   sum <i> in LoopIdx do x[a, i] == 1;

subto newloopup: forall <a,i> in A*LoopIdx do
   delta[a, i] <= qup[a]*abs(qup[a]) * x[a,i]
   and
   delta[a, i] >= qlo[a]*abs(qlo[a]) * x[a,i];
   
